@@ -1,3 +1,4 @@
+import warnings
 from base64 import b64encode
 from typing import Any
 from typing import cast
@@ -7,6 +8,9 @@ from jsonschema import ValidationError
 from jsonschema.exceptions import (
     _WrappedReferencingError as WrappedReferencingError,
 )
+from jsonschema.validators import Draft202012Validator
+from jsonschema.validators import extend
+from jsonschema.validators import validator_for
 from referencing import Registry
 from referencing import Resource
 from referencing.exceptions import InvalidAnchor
@@ -24,6 +28,9 @@ from openapi_schema_validator import oas30_format_checker
 from openapi_schema_validator import oas30_strict_format_checker
 from openapi_schema_validator import oas31_format_checker
 from openapi_schema_validator import oas32_format_checker
+from openapi_schema_validator._dialects import OAS31_BASE_DIALECT_METASCHEMA
+from openapi_schema_validator._dialects import register_openapi_dialect
+from openapi_schema_validator.validators import OAS31_BASE_DIALECT_ID
 
 
 class TestOAS30ValidatorFormatChecker:
@@ -1113,3 +1120,84 @@ class TestOAS30StrictValidator:
         # Note: "test" is actually valid base64, so use "not base64" which is not
         with pytest.raises(ValidationError, match="is not a 'binary'"):
             validator.validate("not base64")
+
+
+class TestValidatorForDiscovery:
+    def test_oas31_base_dialect_resolves_to_oas31_validator(self):
+        schema = {"$schema": OAS31_BASE_DIALECT_ID}
+
+        validator_class = validator_for(schema)
+
+        assert validator_class is OAS31Validator
+
+    def test_oas31_base_dialect_discovery_has_no_deprecation_warning(self):
+        schema = {"$schema": OAS31_BASE_DIALECT_ID}
+
+        with warnings.catch_warnings(record=True) as caught:
+            warnings.simplefilter("always")
+            validator_for(schema)
+
+        assert not any(
+            issubclass(warning.category, DeprecationWarning)
+            for warning in caught
+        )
+
+    def test_oas31_base_dialect_keeps_oas_keyword_behavior(self):
+        schema = {
+            "$schema": OAS31_BASE_DIALECT_ID,
+            "type": "object",
+            "required": ["kind"],
+            "properties": {"kind": {"type": "string"}},
+            "discriminator": {"propertyName": "kind"},
+            "xml": {"name": "Pet"},
+            "example": {"kind": "cat"},
+        }
+
+        validator_class = validator_for(schema)
+        validator = validator_class(
+            schema, format_checker=oas31_format_checker
+        )
+
+        result = validator.validate({"kind": "cat"})
+
+        assert result is None
+
+    def test_draft_2020_12_discovery_is_unchanged(self):
+        schema = {"$schema": "https://json-schema.org/draft/2020-12/schema"}
+
+        validator_class = validator_for(schema)
+
+        assert validator_class is Draft202012Validator
+
+    def test_openapi_dialect_registration_is_idempotent(self):
+        register_openapi_dialect(
+            validator=OAS31Validator,
+            dialect_id=OAS31_BASE_DIALECT_ID,
+            version_name="oas31",
+            metaschema=OAS31_BASE_DIALECT_METASCHEMA,
+        )
+        register_openapi_dialect(
+            validator=OAS31Validator,
+            dialect_id=OAS31_BASE_DIALECT_ID,
+            version_name="oas31",
+            metaschema=OAS31_BASE_DIALECT_METASCHEMA,
+        )
+
+        validator_class = validator_for({"$schema": OAS31_BASE_DIALECT_ID})
+
+        assert validator_class is OAS31Validator
+
+    def test_openapi_dialect_registration_does_not_replace_validator(self):
+        another_oas31_validator = extend(OAS31Validator, {})
+
+        registered_validator = register_openapi_dialect(
+            validator=another_oas31_validator,
+            dialect_id=OAS31_BASE_DIALECT_ID,
+            version_name="oas31",
+            metaschema=OAS31_BASE_DIALECT_METASCHEMA,
+        )
+
+        assert registered_validator is OAS31Validator
+        assert (
+            validator_for({"$schema": OAS31_BASE_DIALECT_ID}) is OAS31Validator
+        )
