@@ -1,3 +1,5 @@
+from unittest.mock import patch
+
 import pytest
 
 from openapi_schema_validator import validate
@@ -32,3 +34,10 @@ def test_validate_does_not_mutate_schema(schema):
     original_schema = schema.copy()
     validate({"email": "foo@bar.com"}, schema)
     assert schema == original_schema
+
+
+def test_validate_does_not_fetch_remote_metaschemas(schema):
+    with patch("urllib.request.urlopen") as urlopen:
+        validate({"email": "foo@bar.com"}, schema)
+
+    urlopen.assert_not_called()
